'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createPluginApi;

var _errorhub = require('../errorhub');

var errorhub = _interopRequireWildcard(_errorhub);

var _clone = require('./clone');

var _clone2 = _interopRequireDefault(_clone);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var _plugins = [];

window._pluginMacroLoaderGlobal = window._pluginMacroLoaderGlobal || {};

function createPluginApi() {

	function _PluginApi(pluginThis, hotReload) {
		if (Object.defineProperty) {
			Object.defineProperty(this, '_context', {
				value: {},
				enumerable: false,
				writable: false
			});

			Object.defineProperty(this, '_spawn', {
				value: [],
				enumerable: false,
				writable: false
			});

			Object.defineProperty(this, 'global', {
				enumerable: false,
				get: function get() {
					return window._pluginMacroLoaderGlobal[pluginThis.name];
				},
				set: function set(value) {
					window._pluginMacroLoaderGlobal[pluginThis.name] = value;
				}
			});
		} else {
			this._context = {};
			this._spawn = [];
			this.global = undefined;
		}

		_plugins.push(this);
		this._PluginApi = _PluginApi;
		this._pluginThis = pluginThis;
		this._hotReload = hotReload;
	}

	_PluginApi.prototype = PluginApi.prototype;

	for (var _len = arguments.length, apiParents = Array(_len), _key = 0; _key < _len; _key++) {
		apiParents[_key] = arguments[_key];
	}

	_PluginApi._apiParents = apiParents;
	_PluginApi._apiList = {};
	_PluginApi._apiDestroy = {};
	_PluginApi._apiInitialState = {};

	_PluginApi.mock = function () {
		for (var _len2 = arguments.length, keys = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
			keys[_key2] = arguments[_key2];
		}

		keys.forEach(function (key) {
			return PluginApi.prototype[key] = PluginApi.prototype[key] || apiAccessor(key);
		});
	};

	_PluginApi.hotReload = function (name) {
		var apiNew = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

		var _apiNew = _extends({}, apiNew);
		delete _apiNew.destroy;
		delete _apiNew.initialState;

		var exists = [];
		Object.keys(_PluginApi._apiList).filter(function (apiName) {
			return apiName !== name;
		}).map(function (apiName) {
			return _PluginApi._apiList[apiName];
		}).forEach(function (apiSibling) {
			Object.keys(_apiNew).forEach(function (key) {
				if (apiSibling[key] !== undefined) {
					exists.push(key);
				}
			});
		});

		if (exists.length) {
			throw new Error("PluginApi methods '" + exists.join(', ') + "' already exist.");
		}

		_PluginApi._apiList[name] = _apiNew;

		Object.keys(_apiNew).forEach(function (key) {
			PluginApi.prototype[key] = PluginApi.prototype[key] || apiAccessor(key);
		});

		hotReload(name);

		_PluginApi._apiDestroy[name] = apiNew.destroy;
		_PluginApi._apiInitialState[name] = apiNew.initialState;
	};

	return _PluginApi;
}

function apiAccessor(key) {
	return function () {
		try {
			var pluginApiList = [this._PluginApi].concat(this._PluginApi._apiParents);

			for (var i = 0; i < pluginApiList.length; i++) {
				var apiList = pluginApiList[i]._apiList;

				var _iteratorNormalCompletion = true;
				var _didIteratorError = false;
				var _iteratorError = undefined;

				try {
					for (var _iterator = Object.keys(apiList)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
						var name = _step.value;

						if (key in apiList[name]) {
							var apiFn = apiList[name][key].default || apiList[name][key];

							this._context[name] = this._context[name] || [];

							var initialState = pluginApiList[i]._apiInitialState[name];

							if (initialState) {
								if (this._context[name][i] === undefined) {
									this._context[name][i] = typeof initialState === 'function' ? initialState() : (0, _clone2.default)(initialState);
								}

								return apiFn.call.apply(apiFn, [this._pluginThis, this._context[name][i]].concat(Array.prototype.slice.call(arguments)));
							} else {
								var result = apiFn.call.apply(apiFn, [this._pluginThis, this._context[name][i]].concat(Array.prototype.slice.call(arguments)));

								if (this._context[name][i] === undefined) {
									this._context[name][i] = result;
								}
							}
						}
					}
				} catch (err) {
					_didIteratorError = true;
					_iteratorError = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion && _iterator.return) {
							_iterator.return();
						}
					} finally {
						if (_didIteratorError) {
							throw _iteratorError;
						}
					}
				}
			}
		} catch (e) {
			errorhub.dispatch(errorhub.ERROR.APICALL, 'Failed apicall', e, key);
		}
	};
}

function PluginApi() {}

PluginApi.prototype.eachApi = function (name, fn) {
	(this._context[name] || []).forEach(fn);
	this._spawn.forEach(function (api) {
		return api.eachApi(name, fn);
	});
};

PluginApi.prototype.destroy = function () {
	var _this = this;

	this._spawn.forEach(function (api) {
		return api.destroy();
	});

	if (_plugins.indexOf(this) !== -1) {
		_plugins.splice(_plugins.indexOf(this), 1);
	}

	Object.keys(this._context).forEach(function (name) {
		for (var i = 0; i < _this._context[name].length; i++) {
			if (_this._context[name][i] !== undefined) {
				var _apiDestroy = i ? _this._PluginApi._apiParents[i - 1]._apiDestroy : _this._PluginApi._apiDestroy;

				if (typeof _apiDestroy[name] === 'function') {
					_apiDestroy[name](_this._context[name][i]);
				}
			}
		}
	});
};

PluginApi.prototype.spawn = function (pluginThis) {
	var _PluginApi = this._PluginApi;
	var api = new _PluginApi(pluginThis, this.hotReload);
	this._spawn.push(api);
	return api;
};

function hotReload(name) {
	var error = null;
	_plugins.filter(function (pluginApi) {
		return name in pluginApi._context;
	}).forEach(function (_ref) {
		var _hotReload = _ref._hotReload;

		try {
			_hotReload && _hotReload();
		} catch (e) {
			error = e;
		}
	});
}