'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createState;

var _lodash = require('./lodash');

var _clone = require('./clone');

var _clone2 = _interopRequireDefault(_clone);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var properties = ['initial', 'reset', 'resetInitial', 'persist', 'isDirty', 'isDirtyInitial'];
var propertiesFactory = [createInitial, createReset, createResetInitial, createPersist, createIsDirty, createIsDirtyInitial];

function createState() {
	var pointers = [];
	var states = [];
	var state = function state() {
		var _arguments = arguments;

		var found = void 0;
		var _iteratorNormalCompletion = true;
		var _didIteratorError = false;
		var _iteratorError = undefined;

		try {
			for (var _iterator = pointers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
				var pointer = _step.value;

				if (pointer.length !== arguments.length) {
					continue;
				} else if (pointer.filter(function (val, i) {
					return val !== _arguments[i];
				}).length) {
					continue;
				}

				return states[pointers.indexOf(pointer)];
			}
		} catch (err) {
			_didIteratorError = true;
			_iteratorError = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion && _iterator.return) {
					_iterator.return();
				}
			} finally {
				if (_didIteratorError) {
					throw _iteratorError;
				}
			}
		}

		pointers.push([].concat(Array.prototype.slice.call(arguments)));
		states.push(createState());
		return states[states.length - 1];
	};

	for (var _i = 0; _i < properties.length; _i++) {
		if (Object.defineProperty) {
			Object.defineProperty(state, properties[_i], {
				value: propertiesFactory[_i](state),
				enumerable: false,
				writable: false
			});
		} else {
			state[properties[_i]] = propertiesFactory[_i](state);
		}
	}

	state.__proto__ = Object.prototype;
	return state;
}

function createInitial(state) {
	var initial = function initial(initialState, zeroState) {
		initialState = initialState || {};
		zeroState = zeroState || {};

		checkState(initialState);
		checkState(zeroState);

		copyInto(initial.initialState, (0, _clone2.default)(initialState));
		copyInto(initial.zeroState, (0, _clone2.default)(zeroState));

		var newState = _extends({}, initial.zeroState, initial.initialState, initial.persistState);

		extendObject(state, (0, _clone2.default)(newState), initial.source, initial.pointers, initial.sources);
	};

	initial.pointers = [];
	initial.sources = [];
	initial.source = {};
	initial.persistState = {};
	initial.initialState = {};
	initial.zeroState = {};

	return initial;
}

function createReset(state) {
	return function () {
		var zero = arguments.length <= 0 || arguments[0] === undefined ? false : arguments[0];

		if (zero === false && state.isDirtyInitial()) {
			state.resetInitial();
		} else if (state.isDirty()) {
			copyState(state, _extends({}, state.initial.zeroState, state.initial.initialState, state.initial.persistState, state.initial.zeroState));
		}
	};
}

function createResetInitial(state) {
	return function () {
		copyState(state, _extends({}, state.initial.zeroState, state.initial.initialState, state.initial.persistState));
	};
}

function createPersist(state) {
	return function (persistState) {
		var copyToState = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

		if (persistState === undefined) {
			persistState = {};

			Object.keys(state.initial.initialState).forEach(function (key) {
				persistState[key] = state[key];
			});
		}

		checkState(persistState);

		state.initial.persistState = _extends({}, state.initial.persistState, (0, _clone2.default)(persistState || {}));

		if (copyToState) {
			copyState(state, persistState);
		}
	};
}

function createIsDirty(state) {
	return function () {
		return isDirty(state, [state.initial.zeroState, state.initial.persistState, state.initial.initialState]);
	};
}

function createIsDirtyInitial(state) {
	return function () {
		return isDirty(state, [state.initial.persistState, state.initial.initialState]);
	};
}

function checkState(state) {
	if ((0, _lodash.isPlainObject)(state) === false) {
		throw new Error('Plugin state has to be plain Object.');
	}

	var _iteratorNormalCompletion2 = true;
	var _didIteratorError2 = false;
	var _iteratorError2 = undefined;

	try {
		for (var _iterator2 = properties[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
			var key = _step2.value;

			if (key in state) {
				throw new Error("Plugin state does not support '" + key + "' key.");
			}
		}
	} catch (err) {
		_didIteratorError2 = true;
		_iteratorError2 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion2 && _iterator2.return) {
				_iterator2.return();
			}
		} finally {
			if (_didIteratorError2) {
				throw _iteratorError2;
			}
		}
	}
}

function isDirty(state, states) {
	var _iteratorNormalCompletion3 = true;
	var _didIteratorError3 = false;
	var _iteratorError3 = undefined;

	try {
		for (var _iterator3 = Object.keys(state)[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
			var key = _step3.value;

			states: {
				var _iteratorNormalCompletion4 = true;
				var _didIteratorError4 = false;
				var _iteratorError4 = undefined;

				try {
					for (var _iterator4 = states[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
						var _state = _step4.value;

						if (_state[key] !== undefined) {
							if ((0, _lodash.isEqual)(state[key], _state[key]) === false) {
								return true;
							}

							break states;
						}
					}
				} catch (err) {
					_didIteratorError4 = true;
					_iteratorError4 = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion4 && _iterator4.return) {
							_iterator4.return();
						}
					} finally {
						if (_didIteratorError4) {
							throw _iteratorError4;
						}
					}
				}
			}
		}
	} catch (err) {
		_didIteratorError3 = true;
		_iteratorError3 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion3 && _iterator3.return) {
				_iterator3.return();
			}
		} finally {
			if (_didIteratorError3) {
				throw _iteratorError3;
			}
		}
	}

	return false;
}

function copyInto(obj, state) {
	var _iteratorNormalCompletion5 = true;
	var _didIteratorError5 = false;
	var _iteratorError5 = undefined;

	try {
		for (var _iterator5 = Object.keys(state)[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
			var key = _step5.value;

			obj[key] = state[key];
		}
	} catch (err) {
		_didIteratorError5 = true;
		_iteratorError5 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion5 && _iterator5.return) {
				_iterator5.return();
			}
		} finally {
			if (_didIteratorError5) {
				throw _iteratorError5;
			}
		}
	}
}

function copyState(state, newState) {
	newState = (0, _clone2.default)(newState);

	Object.keys(newState).forEach(function (key) {
		delete state[key];
		extend(key, state, newState, state.initial.source, state.initial.pointers, state.initial.sources);
	});
}

function extendObject(state, initialState, source, pointers, sources) {
	Object.keys(initialState).forEach(function (key) {
		extend(key, state, initialState, source, pointers, sources);
	});

	Object.keys(state).forEach(function (key) {
		if (initialState[key] === undefined && (0, _lodash.isEqual)(state[key], source[key])) {
			delete state[key];
			delete initialState[key];
		}
	});
}

function extendArray(state, initialState, source, pointers, sources) {
	for (var _i2 = 0; _i2 < initialState.length; _i2++) {
		extend(_i2, state, initialState, source, pointers, sources);
	}

	if (state.length > initialState.length && state.length === source.length) {
		var equal = true;
		for (i; i < state.length; i++) {
			equal = equal && (0, _lodash.isEqual)(state[i], source[i]);
		}

		if (equal) {
			state.splice(initialState.length);
			source.splice(initialState.length);
		}
	}
}

function extend(key, state, initialState, source, pointers, sources) {
	var value = initialState[key];
	var index = pointers.indexOf(state[key]);

	if (index !== -1) {
		if ((0, _lodash.isPlainObject)(state[key]) && (0, _lodash.isPlainObject)(value)) {
			extendObject(state[key], value, sources[index], pointers, sources);
		} else if (Array.isArray(state[key]) && Array.isArray(value)) {
			extendArray(state[key], value, sources[index], pointers, sources);
		} else {
			state[key] = undefined;
		}
	}

	if (state[key] === undefined) {
		if ((0, _lodash.isPlainObject)(value)) {
			var _source = {};
			state[key] = {};
			pointers.push(state[key]);
			sources.push(_source);
			extendObject(state[key], value, _source, pointers, sources);
		} else if (Array.isArray(value)) {
			var _source2 = [];
			state[key] = [];
			pointers.push(state[key]);
			sources.push(_source2);
			extendArray(state[key], value, _source2, pointers, sources);
		} else {
			state[key] = value;
			source[key] = value;
		}
	} else if (state[key] === source[key]) {
		source[key] = value;
		state[key] = value;
	}
}