'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.mockApi = mockApi;
exports.bindApi = bindApi;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * @param {object}
 * @return {function}
 */
function mockApi(PluginApi) {

	/**
  * @param {string...}
  */
	return function () {
		PluginApi.mock.apply(PluginApi, arguments);
	};
}

/**
 * @param {object}
 */
function bindApi(PluginApi) {

	var apiBinder = {};

	/**
  * @param {string}
  * @param {function|object}
  * @return {object}
  */
	return function (name, api) {
		apiBinder[name] = apiBinder[name] || createBinder(PluginApi, name);
		apiBinder[name].hotReload(api);
		return apiBinder[name];
	};
}

function createBinder(PluginApi, name) {
	var hotReloaded = false;

	return {
		hotReload: function hotReload(api) {
			api = api.default || api;

			if (typeof api === 'function') {
				api = _defineProperty({}, name, api);
			}

			hotReloaded = true;
			PluginApi.hotReload(name, api);
		},
		willDispose: function willDispose() {
			hotReloaded = false;

			setTimeout(function () {
				if (hotReloaded === false) {
					PluginApi.hotReload(name);
				}
			}, 0);
		}
	};
}