'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.bind = bind;
exports.load = load;
exports.unloadPlugins = unloadPlugins;
exports.findPlugins = findPlugins;
exports.command = command;
exports.commandAll = commandAll;

var _errorhub = require('../errorhub');

var errorhub = _interopRequireWildcard(_errorhub);

var _internalSelector = require('./internalSelector');

var _scripts = require('./scripts');

var _clone = require('../lib/clone');

var _clone2 = _interopRequireDefault(_clone);

var _resolveDataAttribute = require('../lib/resolveDataAttribute');

var _resolveDataAttribute2 = _interopRequireDefault(_resolveDataAttribute);

var _contains = require('../lib/contains');

var _contains2 = _interopRequireDefault(_contains);

var _destroy = require('../lib/destroy');

var _state = require('../lib/state');

var _state2 = _interopRequireDefault(_state);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var _binders = [];
var _loaded = [];

(0, _internalSelector.bindInternalSelector)('[data-plugin]:not(noscript)', function ($element, options) {
	(0, _resolveDataAttribute2.default)($element, 'data-plugin', function (_ref) {
		var pluginName = _ref.pluginName;
		var pluginId = _ref.pluginId;
		var name = _ref.name;
		var createData = _ref.createData;

		var pluginDef = getPluginByName(pluginName);

		if (pluginDef) {
			(function () {
				var _pluginDef = _slicedToArray(pluginDef, 2);

				var createInstance = _pluginDef[0];
				var plugin = _pluginDef[1];

				var state = (0, _state2.default)();
				var data = createData();
				var pluginArguments = [$element, data, state, pluginId];
				var pluginArgumentsObj = { $element: $element, $root: $element, data: data, state: state, pluginId: pluginId };

				var reloadData = function reloadData() {
					var data = createData();
					pluginArguments[1] = data;
					pluginArgumentsObj.data = data;
				};

				var pluginObj = { $element: $element, plugin: plugin, state: state, reloadData: reloadData, pluginArguments: pluginArguments, pluginArgumentsObj: pluginArgumentsObj, pluginName: pluginName, pluginId: pluginId, name: name, options: options };
				_loaded.push(createInstance(pluginObj, plugin));
			})();
		}
	});
});

function getPluginByName(pluginName) {
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = _binders[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var _step$value = _slicedToArray(_step.value, 2);

			var createInstance = _step$value[0];
			var plugins = _step$value[1];

			if (plugins[pluginName]) {
				return [createInstance, plugins[pluginName]];
			}
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}
}

function bind(createInstance) {

	var _plugins = {};
	var internalBinders = {};
	_binders.push([createInstance, _plugins]);

	function createBinder(createInstance, name) {
		var hotReloaded = false;

		return {
			plugins: function plugins() {
				return findPlugins(function (_ref2) {
					var _name = _ref2.name;
					var plugin = _ref2.plugin;
					return _name === name && plugin === _plugins[name];
				});
			},
			hotReload: function hotReload(plugin) {
				if (!plugin) {
					return this.willDispose();
				}

				hotReloaded = true;

				var oldPlugin = _plugins[name];
				var newPlugin = plugin && plugin.default || plugin;

				_plugins[name] = newPlugin;
				_hotReload(name, oldPlugin, newPlugin);
			},
			willDispose: function willDispose() {
				hotReloaded = false;

				setTimeout(function () {
					if (hotReloaded === false) {
						var oldPlugin = _plugins[name];
						delete _plugins[name];
						delete internalBinders[name];
						_hotReload(name, oldPlugin);
					}
				}, 0);
			}
		};
	}

	/**
  * @param {string}
  * @param {function}
  * @return {object}
  */
	function bind(name, plugin) {
		internalBinders[name] = internalBinders[name] || createBinder(createInstance, name);
		internalBinders[name].hotReload(plugin);
		return internalBinders[name];
	}

	/**
  * @param {string}
  * @param {function}
  * @param {mixed}
  * @return {function}
  */
	function bindSelector(selector, plugin, _data) {
		var pluginName = '$' + selector;
		return _bindSelector(createInstance, pluginName, selector, plugin, function ($element, state) {
			var data = (0, _clone2.default)(_data);
			var pluginArguments = data === undefined ? [$element, state] : [$element, data, state];
			var pluginArgumentsObj = { $element: $element, $root: $element, state: state, data: data };
			return { pluginArguments: pluginArguments, pluginArgumentsObj: pluginArgumentsObj };
		});
	}

	/**
  * @param {string}
  * @param {function}
  */
	function bindAttribute(attr, plugin) {
		var selector = '[' + attr + ']';
		var pluginName = selector;
		return _bindSelector(createInstance, pluginName, selector, plugin, function ($element, state) {
			var pvar = $element.getAttribute(attr) || '';
			var data = (0, _scripts.loadData)(pvar);
			var pluginArguments = [$element, data, state];
			var pluginArgumentsObj = { $element: $element, $root: $element, data: data, state: state };
			var reloadData = function reloadData() {
				var pvar = $element.getAttribute(attr) || '';
				var data = (0, _scripts.loadData)(pvar);
				pluginArguments[1] = data;
				pluginArgumentsObj.data = data;
			};

			return { pluginArguments: pluginArguments, pluginArgumentsObj: pluginArgumentsObj, reloadData: reloadData };
		});
	}

	return { bind: bind, bindSelector: bindSelector, bindAttribute: bindAttribute };
}

/**
 * @param {string}
 * @param {string}
 * @param {function}
 * @param {function}
 */
function _bindSelector(createInstance, pluginName, selector, plugin, createArguments) {
	plugin = plugin.default || plugin;

	var loader = (0, _internalSelector.bindInternalSelector)(selector, function ($element) {
		var state = (0, _state2.default)();

		var _createArguments = createArguments($element, state);

		var pluginArguments = _createArguments.pluginArguments;
		var pluginArgumentsObj = _createArguments.pluginArgumentsObj;
		var reloadData = _createArguments.reloadData;

		var pluginObj = { $element: $element, plugin: plugin, state: state, reloadData: reloadData, pluginArguments: pluginArguments, pluginArgumentsObj: pluginArgumentsObj, pluginName: pluginName, options: {} };
		_loaded.push(createInstance(pluginObj, plugin));
	});

	return {
		plugins: function plugins() {
			return findPlugins(function (_ref3) {
				var _pluginName = _ref3.pluginName;
				var _plugin = _ref3.plugin;
				return _pluginName === pluginName && _plugin === plugin;
			});
		},
		hotReload: function hotReload(newPlugin) {
			if (!newPlugin) {
				return this.willDispose();
			}

			var oldPlugin = plugin;
			plugin = newPlugin.default || newPlugin;
			_hotReload(pluginName, oldPlugin, plugin);
		},
		willDispose: function willDispose() {
			var oldPlugin = plugin;
			plugin = undefined;
			_hotReload(pluginName, oldPlugin);
			loader();
		}
	};
}

/**
 * @param {string}
 * @param {function}
 * @param {function}
 */
function _hotReload(name, oldPlugin) {
	var newPlugin = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];

	findPlugins(function (_ref4) {
		var plugin = _ref4.plugin;
		var pluginName = _ref4.pluginName;
		return pluginName === name && plugin === oldPlugin;
	}).forEach(function (pluginObj) {
		pluginObj.hotReload(newPlugin);
	});
}

/**
 * @param {Element}
 * @param {Object}
 * @return {Array}
 */
function load($dom) {
	var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

	(0, _internalSelector.loadInternalSelector)($dom, options);
}

/**
 * @param {array}
 */
function unloadPlugins(plugins) {
	plugins.forEach(function (pluginObj) {
		if (_loaded.indexOf(pluginObj) !== -1) {
			_loaded.splice(_loaded.indexOf(pluginObj), 1);
		}

		(0, _destroy.destroyPlugin)(pluginObj, false);
	});
}

/**
 * @param {Element|function}
 * @return {array}
 */
function findPlugins(cond) {
	if (typeof cond === 'function') {
		return _loaded.filter(function (val) {
			return cond(val);
		});
	}

	return findPlugins(function (_ref5) {
		var $element = _ref5.$element;
		return (0, _contains2.default)(cond, $element);
	});
}

/**
 * @return {mixed}
 */
function command(name, command) {
	for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
		args[_key - 2] = arguments[_key];
	}

	var result = commandAll.apply(undefined, [name, command].concat(args));
	return result.length && result[0] || null;
}

/**
 * @param {function|string|Element}
 * @param {string}
 * @param {mixed..}
 * @return {Array}
 */
function commandAll(name, command) {
	for (var _len2 = arguments.length, args = Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
		args[_key2 - 2] = arguments[_key2];
	}

	var _arguments = arguments;

	return _loaded.filter(function (val) {
		return Object.values(val).indexOf(name) !== -1;
	}).filter(function (_ref6) {
		var instance = _ref6.instance;
		return instance && instance[command];
	}).map(function (_ref7) {
		var instance = _ref7.instance;

		try {
			return instance[command].apply(instance, args);
		} catch (e) {
			errorhub.dispatch(errorhub.ERROR.COMMAND, 'Failed commandAll', e, _arguments);
		}
	});
}