'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.loader = loader;
exports.loaderGlobal = loaderGlobal;
exports.default = create;

var _notify = require('./lib/notify');

var _notify2 = _interopRequireDefault(_notify);

var _loader2 = require('./loader');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function loader(parentModule, fn) {
	createHotLoader(_loader2.loader, parentModule, fn);
}

function loaderGlobal(parentModule, fn) {
	createHotLoader(_loader2.loaderGlobal, parentModule, fn);
}

function createHotLoader(loader, parentModule, fn) {
	var hotReload = create(parentModule);

	loader(function (binders) {
		var bindApi = binders.bindApi;
		var bindSystem = binders.bindSystem;
		var bindGlobal = binders.bindGlobal;
		var bind = binders.bind;
		var bindSelector = binders.bindSelector;
		var bindAttribute = binders.bindAttribute;


		fn(_extends({}, binders, {
			bindApi: patchBinder(bindApi),
			bindSystem: patchBinder(bindSystem),
			bindGlobal: patchBinder(bindGlobal),
			bind: patchBinder(bind),
			bindSelector: patchBinder(bindSelector),
			bindAttribute: patchBinder(bindAttribute),
			hotBindApi: hotReload.bind(undefined, bindApi),
			hotBindSystem: hotReload.bind(undefined, bindSystem),
			hotBindGlobal: hotReload.bind(undefined, bindGlobal),
			hotBind: hotReload.bind(undefined, bind),
			hotBindSelector: hotReload.bind(undefined, bindSelector),
			hotBindAttribute: hotReload.bind(undefined, bindAttribute)
		}));
	});
}

function patchBinder(binder) {
	return function (name, plugin) {
		if (typeof name === 'string' && (typeof plugin === 'undefined' ? 'undefined' : _typeof(plugin)) === 'object') {
			plugin = plugin.default || plugin;
		} else if ((typeof name === 'undefined' ? 'undefined' : _typeof(name)) === 'object') {
			name = name.default || name;
		}

		for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
			args[_key - 2] = arguments[_key];
		}

		return binder.apply(undefined, [name, plugin].concat(args));
	};
}

function create(parentModule) {
	var plugins = [];

	if (parentModule.hot) {
		parentModule.hot.addDisposeHandler(function () {
			plugins.forEach(function (plugin) {
				return plugin.willDispose();
			});
			plugins.splice(0, plugins.length);
			(0, _notify2.default)("Plugins have been disposed.");
		});

		parentModule.hot.accept();
	}

	return function hotReload(binder, name, plugin) {
		for (var _len2 = arguments.length, args = Array(_len2 > 3 ? _len2 - 3 : 0), _key2 = 3; _key2 < _len2; _key2++) {
			args[_key2 - 3] = arguments[_key2];
		}

		if (typeof name === 'number') {
			args.unshift(plugin);
			plugin = name;
			name = '';
		}

		var pluginName = binder.name + "('" + name + "'')";
		var pluginObj = name ? binder.apply(undefined, [name, __webpack_require__(plugin)['default']].concat(args)) : binder.apply(undefined, [__webpack_require__(plugin)['default']].concat(args));

		plugins.push(pluginObj);

		if (module.hot) {
			module.hot.accept(plugin, function () {
				var $update = document.createElement('div');
				$update.textContent = "Plugin " + pluginName + " update detected. Reloading..";
				$update.style.padding = '6px 3px';

				(0, _notify2.default)($update);

				try {
					pluginObj.hotReload(__webpack_require__(plugin)['default']);
					$update.style.backgroundColor = "#33cc33";
					$update.textContent = "Plugin " + pluginName + " reloaded.";
				} catch (e) {
					$update.style.backgroundColor = "#cc0000";
					$update.textContent = "Plugin " + pluginName + " update failed (" + e.message + ").";
					console.error(e, e.stack);
				}
			});
		}

		return pluginObj;
	};
}